@@ -45,7 +45,7 @@
 
 struct GdmStaticDisplayPrivate
 {
-        gpointer dummy;
+        gboolean enable_timed_login;
 };
 
 enum {
@@ -121,6 +121,9 @@ gdm_static_display_unmanage (GdmDisplay *display)
 {
         g_return_val_if_fail (GDM_IS_DISPLAY (display), FALSE);
 
+        /* Only allow timed/automatic login once */
+        GDM_STATIC_DISPLAY (display)->priv->enable_timed_login = FALSE;
+
         GDM_DISPLAY_CLASS (gdm_static_display_parent_class)->unmanage (display);
 
         return TRUE;
@@ -152,6 +155,21 @@ gdm_static_display_get_property (GObject    *object,
         }
 }
 
+static void
+gdm_static_display_get_timed_login_details (GdmDisplay *display,
+                                            gboolean   *enabledp,
+                                            char      **usernamep,
+                                            int        *delayp)
+{
+        if (GDM_STATIC_DISPLAY (display)->priv->enable_timed_login) {
+                GDM_DISPLAY_CLASS (gdm_static_display_parent_class)->get_timed_login_details (display, enabledp, usernamep, delayp);
+        } else {
+                *enabledp = FALSE;
+                *usernamep = g_strdup ("");
+                *delayp = 0;
+        }
+}
+
 static void
 gdm_static_display_class_init (GdmStaticDisplayClass *klass)
 {
@@ -168,6 +186,7 @@ gdm_static_display_class_init (GdmStaticDisplayClass *klass)
         display_class->manage = gdm_static_display_manage;
         display_class->finish = gdm_static_display_finish;
         display_class->unmanage = gdm_static_display_unmanage;
+        display_class->get_timed_login_details = gdm_static_display_get_timed_login_details;
 
         g_type_class_add_private (klass, sizeof (GdmStaticDisplayPrivate));
 
@@ -179,6 +198,8 @@ gdm_static_display_init (GdmStaticDisplay *static_display)
 {
 
         static_display->priv = GDM_STATIC_DISPLAY_GET_PRIVATE (static_display);
+
+        static_display->priv->enable_timed_login = TRUE;
 }
 
 static void
