@@ -2,6 +2,7 @@
  *
  * Copyright (C) 2007 Ray Strode <rstrode@redhat.com>
  * Copyright (C) 2007 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2008 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -67,6 +68,7 @@ struct GdmChooserWidgetPrivate
 
         GdkPixbuf                *is_in_use_pixbuf;
 
+        /* row for the list_store model */
         GtkTreeRowReference      *active_row;
         GtkTreeRowReference      *separator_row;
 
@@ -87,12 +89,10 @@ struct GdmChooserWidgetPrivate
 
         guint                    update_idle_id;
         guint                    timer_animation_timeout_id;
-        guint                    activate_idle_id;
 
         guint32                  should_hide_inactive_items : 1;
         guint32                  emit_activated_after_resize_animation : 1;
         guint32                  was_fully_grown : 1;
-        guint32                  activate_on_one_item : 1;
 
         GdmChooserWidgetPosition separator_position;
         GdmChooserWidgetState    state;
@@ -110,6 +110,7 @@ enum {
 enum {
         ACTIVATED = 0,
         DEACTIVATED,
+        LOADED,
         NUMBER_OF_SIGNALS
 };
 
@@ -124,6 +125,7 @@ static void     update_timer_from_time         (GdmChooserWidget    *widget,
                                                 double               now);
 
 G_DEFINE_TYPE (GdmChooserWidget, gdm_chooser_widget, GTK_TYPE_ALIGNMENT)
+
 enum {
         CHOOSER_IMAGE_COLUMN = 0,
         CHOOSER_NAME_COLUMN,
@@ -160,8 +162,11 @@ find_item (GdmChooserWidget *widget,
         do {
                 char *item_id;
 
-                gtk_tree_model_get (model, iter,
-                                    CHOOSER_ID_COLUMN, &item_id, -1);
+                gtk_tree_model_get (model,
+                                    iter,
+                                    CHOOSER_ID_COLUMN,
+                                    &item_id,
+                                    -1);
 
                 g_assert (item_id != NULL);
 
@@ -252,6 +257,76 @@ gdm_chooser_widget_update_foreach_item (GdmChooserWidget           *widget,
                                 &fdata);
 }
 
+static void
+translate_list_path_to_view_path (GdmChooserWidget  *widget,
+                                  GtkTreePath      **path)
+{
+        GtkTreePath *filtered_path;
+        GtkTreePath *sorted_path;
+
+        /* the child model is the source for the filter */
+        filtered_path = gtk_tree_model_filter_convert_child_path_to_path (widget->priv->model_filter,
+                                                                          *path);
+        sorted_path = gtk_tree_model_sort_convert_child_path_to_path (widget->priv->model_sorter,
+                                                                      filtered_path);
+        gtk_tree_path_free (filtered_path);
+
+        gtk_tree_path_free (*path);
+        *path = sorted_path;
+}
+
+
+static void
+translate_view_path_to_list_path (GdmChooserWidget  *widget,
+                                  GtkTreePath      **path)
+{
+        GtkTreePath *filtered_path;
+        GtkTreePath *list_path;
+
+        /* the child model is the source for the filter */
+        filtered_path = gtk_tree_model_sort_convert_path_to_child_path (widget->priv->model_sorter,
+                                                                        *path);
+
+        list_path = gtk_tree_model_filter_convert_path_to_child_path (widget->priv->model_filter,
+                                                                      filtered_path);
+        gtk_tree_path_free (filtered_path);
+
+        gtk_tree_path_free (*path);
+        *path = list_path;
+}
+
+static GtkTreePath *
+get_list_path_to_active_row (GdmChooserWidget *widget)
+{
+        GtkTreePath *path;
+
+        if (widget->priv->active_row == NULL) {
+                return NULL;
+        }
+
+        path = gtk_tree_row_reference_get_path (widget->priv->active_row);
+        if (path == NULL) {
+                return NULL;
+        }
+
+        return path;
+}
+
+static GtkTreePath *
+get_view_path_to_active_row (GdmChooserWidget *widget)
+{
+        GtkTreePath *path;
+
+        path = get_list_path_to_active_row (widget);
+        if (path == NULL) {
+                return NULL;
+        }
+
+        translate_list_path_to_view_path (widget, &path);
+
+        return path;
+}
+
 static char *
 get_active_item_id (GdmChooserWidget *widget,
                     GtkTreeIter      *iter)
@@ -269,11 +344,18 @@ get_active_item_id (GdmChooserWidget *widget,
                 return NULL;
         }
 
-        path = gtk_tree_row_reference_get_path (widget->priv->active_row);
+        path = get_list_path_to_active_row (widget);
+        if (path == NULL) {
+                return NULL;
+        }
+
         if (gtk_tree_model_get_iter (model, iter, path)) {
-                gtk_tree_model_get (model, iter,
-                                    CHOOSER_ID_COLUMN, &item_id, -1);
-        };
+                gtk_tree_model_get (model,
+                                    iter,
+                                    CHOOSER_ID_COLUMN,
+                                    &item_id,
+                                    -1);
+        }
         gtk_tree_path_free (path);
 
         return item_id;
@@ -287,6 +369,101 @@ gdm_chooser_widget_get_active_item (GdmChooserWidget *widget)
         return get_active_item_id (widget, &iter);
 }
 
+static void
+get_selected_list_path (GdmChooserWidget *widget,
+                        GtkTreePath     **pathp)
+{
+        GtkTreeSelection    *selection;
+        GtkTreeModel        *sort_model;
+        GtkTreeIter          sorted_iter;
+        GtkTreePath         *path;
+
+        path = NULL;
+
+        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget->priv->items_view));
+        if (gtk_tree_selection_get_selected (selection, &sort_model, &sorted_iter)) {
+
+                g_assert (sort_model == GTK_TREE_MODEL (widget->priv->model_sorter));
+
+                path = gtk_tree_model_get_path (sort_model, &sorted_iter);
+
+                translate_view_path_to_list_path (widget, &path);
+        } else {
+                g_debug ("GdmChooserWidget: no rows selected");
+        }
+
+        *pathp = path;
+}
+
+char *
+gdm_chooser_widget_get_selected_item (GdmChooserWidget *widget)
+{
+        GtkTreeIter   iter;
+        GtkTreeModel *model;
+        GtkTreePath  *path;
+        char         *id;
+
+        id = NULL;
+
+        get_selected_list_path (widget, &path);
+
+        model = GTK_TREE_MODEL (widget->priv->list_store);
+
+        if (gtk_tree_model_get_iter (model, &iter, path)) {
+                gtk_tree_model_get (model,
+                                    &iter,
+                                    CHOOSER_ID_COLUMN,
+                                    &id,
+                                    -1);
+        }
+
+        gtk_tree_path_free (path);
+
+        return id;
+}
+
+void
+gdm_chooser_widget_set_selected_item (GdmChooserWidget *widget,
+                                      const char       *id)
+{
+        GtkTreeIter       iter;
+        GtkTreeSelection *selection;
+        GtkTreeModel     *model;
+
+        g_return_if_fail (GDM_IS_CHOOSER_WIDGET (widget));
+
+        g_debug ("GdmChooserWidget: setting selected item '%s'",
+                 id ? id : "(null)");
+
+        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget->priv->items_view));
+
+        model = GTK_TREE_MODEL (widget->priv->list_store);
+
+        if (find_item (widget, id, &iter)) {
+                GtkTreePath  *path;
+
+                path = gtk_tree_model_get_path (model, &iter);
+                translate_list_path_to_view_path (widget, &path);
+
+                gtk_tree_selection_select_path (selection, path);
+
+                gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (widget->priv->items_view),
+                                              path,
+                                              NULL,
+                                              TRUE,
+                                              0.5,
+                                              0.0);
+
+                gtk_tree_view_set_cursor (GTK_TREE_VIEW (widget->priv->items_view),
+                                          path,
+                                          NULL,
+                                          FALSE);
+                gtk_tree_path_free (path);
+        } else {
+                gtk_tree_selection_unselect_all (selection);
+        }
+}
+
 static void
 activate_from_item_id (GdmChooserWidget *widget,
                        const char       *item_id)
@@ -294,19 +471,27 @@ activate_from_item_id (GdmChooserWidget *widget,
         GtkTreeModel *model;
         GtkTreePath  *path;
         GtkTreeIter   iter;
+        char         *path_str;
 
         model = GTK_TREE_MODEL (widget->priv->list_store);
         path = NULL;
 
         if (find_item (widget, item_id, &iter)) {
-                GtkTreePath  *child_path;
+                path = gtk_tree_model_get_path (model, &iter);
 
-                child_path = gtk_tree_model_get_path (model, &iter);
-                path = gtk_tree_model_sort_convert_child_path_to_path (widget->priv->model_sorter, child_path);
-                gtk_tree_path_free (child_path);
+                path_str = gtk_tree_path_to_string (path);
+                g_debug ("GdmChooserWidget: got list path '%s'", path_str);
+                g_free (path_str);
+
+                translate_list_path_to_view_path (widget, &path);
+
+                path_str = gtk_tree_path_to_string (path);
+                g_debug ("GdmChooserWidget: translated to view path '%s'", path_str);
+                g_free (path_str);
         }
 
         if (path == NULL) {
+                g_debug ("GdmChooserWidget: unable to activate - path for item '%s' not found", item_id);
                 return;
         }
 
@@ -360,43 +545,6 @@ set_frame_text (GdmChooserWidget *widget,
         }
 }
 
-static void
-translate_base_path_to_sorted_path (GdmChooserWidget  *widget,
-                                    GtkTreePath      **path)
-{
-        GtkTreePath *filtered_path;
-        GtkTreePath *sorted_path;
-
-        filtered_path =
-            gtk_tree_model_filter_convert_child_path_to_path (widget->priv->model_filter, *path);
-        sorted_path = gtk_tree_model_sort_convert_child_path_to_path (widget->priv->model_sorter,
-                                                                      filtered_path);
-        gtk_tree_path_free (filtered_path);
-
-        gtk_tree_path_free (*path);
-        *path = sorted_path;
-}
-
-static GtkTreePath *
-get_path_to_active_row (GdmChooserWidget *widget)
-{
-        GtkTreePath *path;
-
-        if (widget->priv->active_row == NULL) {
-                return NULL;
-        }
-
-        path= gtk_tree_row_reference_get_path (widget->priv->active_row);
-
-        if (path == NULL) {
-                return NULL;
-        }
-
-        translate_base_path_to_sorted_path (widget, &path);
-
-        return path;
-}
-
 static void
 on_shrink_animation_step (GdmScrollableWidget *scrollable_widget,
                           double               progress,
@@ -406,7 +554,7 @@ on_shrink_animation_step (GdmScrollableWidget *scrollable_widget,
         const double   final_alignment = 0.5;
         double         row_alignment;
 
-        active_row_path = get_path_to_active_row (widget);
+        active_row_path = get_view_path_to_active_row (widget);
         row_alignment = widget->priv->active_row_normalized_position + progress * (final_alignment - widget->priv->active_row_normalized_position);
 
         gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (widget->priv->items_view),
@@ -503,6 +651,8 @@ on_shrink_animation_complete (GdmScrollableWidget *scrollable_widget,
 {
         g_assert (widget->priv->state == GDM_CHOOSER_WIDGET_STATE_SHRINKING);
 
+        g_debug ("GdmChooserWidget: shrink complete");
+
         widget->priv->active_row_normalized_position = 0.5;
         set_inactive_items_visible (GDM_CHOOSER_WIDGET (widget), FALSE);
         gtk_tree_view_set_enable_search (GTK_TREE_VIEW (widget->priv->items_view), FALSE);
@@ -563,7 +713,7 @@ start_shrink_animation (GdmChooserWidget *widget)
                return;
        }
 
-       active_row_path = get_path_to_active_row (widget);
+       active_row_path = get_view_path_to_active_row (widget);
        active_row_height = get_height_of_row_at_path (widget, active_row_path);
        widget->priv->active_row_normalized_position = get_normalized_position_of_row_at_path (widget, active_row_path);
        gtk_tree_path_free (active_row_path);
@@ -599,19 +749,42 @@ activate_if_one_item (GdmChooserWidget *widget)
 {
         char *id;
 
-        widget->priv->activate_idle_id = 0;
+        g_debug ("GdmChooserWidget: attempting to activate single item");
 
         if (gdm_chooser_widget_get_number_of_items (widget) != 1) {
+                g_debug ("GdmChooserWidget: unable to activate single item - has %d items", gdm_chooser_widget_get_number_of_items (widget));
                 return FALSE;
         }
 
         id = get_first_item (widget);
-        gdm_chooser_widget_set_active_item (widget, id);
-        g_free (id);
+        if (id != NULL) {
+                gdm_chooser_widget_set_active_item (widget, id);
+                g_free (id);
+        }
 
         return FALSE;
 }
 
+static void
+_grab_focus (GtkWidget *widget)
+{
+        GtkWidget *foc_widget;
+
+        foc_widget = GDM_CHOOSER_WIDGET (widget)->priv->items_view;
+        g_debug ("GdmChooserWidget: grabbing focus");
+        if (! GTK_WIDGET_REALIZED (foc_widget)) {
+                g_debug ("GdmChooserWidget: not grabbing focus - not realized");
+                return;
+        }
+
+        if (GTK_WIDGET_HAS_FOCUS (foc_widget)) {
+                g_debug ("GdmChooserWidget: not grabbing focus - already has it");
+                return;
+        }
+
+        gtk_widget_grab_focus (foc_widget);
+}
+
 static void
 on_grow_animation_complete (GdmScrollableWidget *scrollable_widget,
                             GdmChooserWidget    *widget)
@@ -621,7 +794,7 @@ on_grow_animation_complete (GdmScrollableWidget *scrollable_widget,
         widget->priv->was_fully_grown = TRUE;
         gtk_tree_view_set_enable_search (GTK_TREE_VIEW (widget->priv->items_view), TRUE);
 
-        gtk_widget_grab_focus (GTK_WIDGET (widget));
+        _grab_focus (GTK_WIDGET (widget));
 }
 
 static int
@@ -724,7 +897,7 @@ skip_resize_animation (GdmChooserWidget *widget)
                 gtk_tree_view_set_enable_search (GTK_TREE_VIEW (widget->priv->items_view), TRUE);
                 widget->priv->state = GDM_CHOOSER_WIDGET_STATE_GROWN;
                 widget->priv->was_fully_grown = FALSE;
-                gtk_widget_grab_focus (GTK_WIDGET (widget));
+                _grab_focus (GTK_WIDGET (widget));
         }
 }
 
@@ -774,6 +947,8 @@ clear_selection (GdmChooserWidget *widget)
         GtkTreeSelection *selection;
         GtkWidget        *window;
 
+        g_debug ("GdmChooserWidget: clearing selection");
+
         selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget->priv->items_view));
         gtk_tree_selection_unselect_all (selection);
 
@@ -826,9 +1001,11 @@ activate_from_row (GdmChooserWidget    *widget,
         widget->priv->active_row = gtk_tree_row_reference_copy (row);
 
         if (widget->priv->should_hide_inactive_items) {
+                g_debug ("GdmChooserWidget: will emit activated after resize");
                 widget->priv->emit_activated_after_resize_animation = TRUE;
                 gdm_chooser_widget_shrink (widget);
         } else {
+                g_debug ("GdmChooserWidget: emitting activated");
                 g_signal_emit (widget, signals[ACTIVATED], 0);
         }
 }
@@ -842,7 +1019,7 @@ deactivate (GdmChooserWidget *widget)
                 return;
         }
 
-        path = get_path_to_active_row (widget);
+        path = get_view_path_to_active_row (widget);
 
         gtk_tree_row_reference_free (widget->priv->active_row);
         widget->priv->active_row = NULL;
@@ -862,50 +1039,40 @@ void
 gdm_chooser_widget_activate_selected_item (GdmChooserWidget *widget)
 {
         GtkTreeRowReference *row;
-        GtkTreeSelection    *selection;
-        GtkTreeModel        *model;
-        GtkTreeModel        *sort_model;
-        GtkTreeIter          sorted_iter;
         gboolean             is_already_active;
+        GtkTreePath         *path;
+        GtkTreeModel        *model;
 
         row = NULL;
         model = GTK_TREE_MODEL (widget->priv->list_store);
         is_already_active = FALSE;
 
-        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget->priv->items_view));
-        if (gtk_tree_selection_get_selected (selection, &sort_model, &sorted_iter)) {
-                GtkTreePath *sorted_path;
-                GtkTreePath *filtered_path;
-                GtkTreePath *base_path;
+        path = NULL;
 
-                g_assert (sort_model == GTK_TREE_MODEL (widget->priv->model_sorter));
-
-                sorted_path = gtk_tree_model_get_path (sort_model, &sorted_iter);
-                filtered_path =
-                    gtk_tree_model_sort_convert_path_to_child_path (widget->priv->model_sorter,
-                                                                    sorted_path);
-                gtk_tree_path_free (sorted_path);
-                base_path =
-                    gtk_tree_model_filter_convert_path_to_child_path (widget->priv->model_filter,
-                                                                      filtered_path);
-
-                if (widget->priv->active_row != NULL) {
-                        GtkTreePath *active_path;
-
-                        active_path = gtk_tree_row_reference_get_path (widget->priv->active_row);
-
-                        if (gtk_tree_path_compare  (base_path, active_path) == 0) {
-                                is_already_active = TRUE;
-                        }
-                        gtk_tree_path_free (active_path);
-                }
-                g_assert (base_path != NULL);
-                row = gtk_tree_row_reference_new (model, base_path);
-                gtk_tree_path_free (base_path);
+        get_selected_list_path (widget, &path);
+        if (path == NULL) {
+                g_debug ("GdmChooserWidget: no row selected");
+                return;
         }
 
+        if (widget->priv->active_row != NULL) {
+                GtkTreePath *active_path;
+
+                active_path = gtk_tree_row_reference_get_path (widget->priv->active_row);
+
+                if (gtk_tree_path_compare (path, active_path) == 0) {
+                        is_already_active = TRUE;
+                }
+                gtk_tree_path_free (active_path);
+        }
+        g_assert (path != NULL);
+        row = gtk_tree_row_reference_new (model, path);
+        gtk_tree_path_free (path);
+
         if (!is_already_active) {
                 activate_from_row (widget, row);
+        } else {
+                g_debug ("GdmChooserWidget: row is already active");
         }
         gtk_tree_row_reference_free (row);
 }
@@ -916,6 +1083,9 @@ gdm_chooser_widget_set_active_item (GdmChooserWidget *widget,
 {
         g_return_if_fail (GDM_IS_CHOOSER_WIDGET (widget));
 
+        g_debug ("GdmChooserWidget: setting active item '%s'",
+                 id ? id : "(null)");
+
         if (id != NULL) {
                 activate_from_item_id (widget, id);
         } else {
@@ -1029,6 +1199,7 @@ static void
 gdm_chooser_widget_show (GtkWidget *widget)
 {
         skip_resize_animation (GDM_CHOOSER_WIDGET (widget));
+
         GTK_WIDGET_CLASS (gdm_chooser_widget_parent_class)->show (widget);
 }
 
@@ -1062,10 +1233,12 @@ gdm_chooser_widget_focus (GtkWidget        *widget,
          *    2) if it doesn't already have focus, then grab it
          */
         if (GTK_CONTAINER (widget)->focus_child != NULL) {
+                g_debug ("GdmChooserWidget: not focusing - focus child not null");
                 return FALSE;
         }
 
-        gtk_widget_grab_focus (GDM_CHOOSER_WIDGET (widget)->priv->items_view);
+        _grab_focus (widget);
+
         return TRUE;
 }
 
@@ -1076,7 +1249,7 @@ gdm_chooser_widget_focus_in_event (GtkWidget     *widget,
         /* We don't ever want the chooser widget itself to have focus.
          * Focus should always go to the tree view.
          */
-        gtk_widget_grab_focus (GDM_CHOOSER_WIDGET (widget)->priv->items_view);
+        _grab_focus (widget);
 
         return FALSE;
 }
@@ -1097,6 +1270,16 @@ gdm_chooser_widget_class_init (GdmChooserWidgetClass *klass)
         widget_class->focus = gdm_chooser_widget_focus;
         widget_class->focus_in_event = gdm_chooser_widget_focus_in_event;
 
+        signals [LOADED] = g_signal_new ("loaded",
+                                         G_TYPE_FROM_CLASS (object_class),
+                                         G_SIGNAL_RUN_LAST,
+                                         G_STRUCT_OFFSET (GdmChooserWidgetClass, loaded),
+                                         NULL,
+                                         NULL,
+                                         g_cclosure_marshal_VOID__VOID,
+                                         G_TYPE_NONE,
+                                         0);
+
         signals [ACTIVATED] = g_signal_new ("activated",
                                             G_TYPE_FROM_CLASS (object_class),
                                             G_SIGNAL_RUN_LAST,
@@ -1145,6 +1328,11 @@ on_row_activated (GtkTreeView          *tree_view,
                   GtkTreeViewColumn    *tree_column,
                   GdmChooserWidget     *widget)
 {
+        char *path_str;
+
+        path_str = gtk_tree_path_to_string (tree_path);
+        g_debug ("GdmChooserWidget: row activated '%s'", path_str ? path_str : "(null)");
+        g_free (path_str);
         gdm_chooser_widget_activate_selected_item (widget);
 }
 
@@ -1390,8 +1578,7 @@ add_separator (GdmChooserWidget *widget)
                                            &iter, 0,
                                            CHOOSER_ID_COLUMN, "-", -1);
         path = gtk_tree_model_get_path (model, &iter);
-        widget->priv->separator_row =
-            gtk_tree_row_reference_new (model, path);
+        widget->priv->separator_row = gtk_tree_row_reference_new (model, path);
         gtk_tree_path_free (path);
 }
 
@@ -1544,6 +1731,23 @@ search_equal_func (GtkTreeModel     *model,
         return ret;
 }
 
+static void
+on_selection_changed (GtkTreeSelection *selection,
+                      GdmChooserWidget *widget)
+{
+        GtkTreePath *path;
+
+        get_selected_list_path (widget, &path);
+        if (path != NULL) {
+                char *path_str;
+                path_str = gtk_tree_path_to_string (path);
+                g_debug ("GdmChooserWidget: selection change to list path '%s'", path_str);
+                g_free (path_str);
+        } else {
+                g_debug ("GdmChooserWidget: selection cleared");
+        }
+}
+
 static void
 gdm_chooser_widget_init (GdmChooserWidget *widget)
 {
@@ -1598,6 +1802,8 @@ gdm_chooser_widget_init (GdmChooserWidget *widget)
         selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget->priv->items_view));
         gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);
 
+        g_signal_connect (selection, "changed", G_CALLBACK (on_selection_changed), widget);
+
         g_assert (NUMBER_OF_CHOOSER_COLUMNS == 11);
         widget->priv->list_store = gtk_list_store_new (NUMBER_OF_CHOOSER_COLUMNS,
                                                        GDK_TYPE_PIXBUF,
@@ -1637,11 +1843,6 @@ gdm_chooser_widget_init (GdmChooserWidget *widget)
 
         /* IMAGE COLUMN */
         renderer = gtk_cell_renderer_pixbuf_new ();
-#if 0
-        gtk_cell_renderer_set_fixed_size (renderer,
-                                          GDM_CHOOSER_WIDGET_DEFAULT_ICON_SIZE,
-                                          GDM_CHOOSER_WIDGET_DEFAULT_ICON_SIZE);
-#endif
         column = gtk_tree_view_column_new ();
         gtk_tree_view_column_pack_start (column, renderer, FALSE);
         gtk_tree_view_append_column (GTK_TREE_VIEW (widget->priv->items_view), column);
@@ -1854,10 +2055,6 @@ gdm_chooser_widget_add_item (GdmChooserWidget *widget,
                                            -1);
 
         move_cursor_to_top (widget);
-
-        if (widget->priv->activate_on_one_item) {
-                activate_if_one_item (widget);
-        }
 }
 
 void
@@ -1906,10 +2103,6 @@ gdm_chooser_widget_remove_item (GdmChooserWidget *widget,
         gtk_list_store_remove (widget->priv->list_store, &iter);
 
         move_cursor_to_top (widget);
-
-        if (widget->priv->activate_on_one_item) {
-                activate_if_one_item (widget);
-        }
 }
 
 gboolean
@@ -2277,14 +2470,9 @@ gdm_chooser_widget_get_number_of_items (GdmChooserWidget *widget)
 }
 
 void
-gdm_chooser_widget_activate_on_one_item (GdmChooserWidget *widget,
-                                         gboolean          should_activate)
+gdm_chooser_widget_activate_if_one_item (GdmChooserWidget *widget)
 {
-        widget->priv->activate_on_one_item = should_activate;
-
-        if (widget->priv->activate_on_one_item) {
-                activate_if_one_item (widget);
-        }
+        activate_if_one_item (widget);
 }
 
 void
@@ -2296,3 +2484,9 @@ gdm_chooser_widget_propagate_pending_key_events (GdmChooserWidget *widget)
 
         gdm_scrollable_widget_replay_queued_key_events (GDM_SCROLLABLE_WIDGET (widget->priv->scrollable_widget));
 }
+
+void
+gdm_chooser_widget_loaded (GdmChooserWidget *widget)
+{
+        g_signal_emit (widget, signals[LOADED], 0);
+}
